@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 #
-# Modified work Copyright (c) 2017-2019 Science and Technology
+# Modified work Copyright (c) 2017-2020 Science and Technology
 # Facilities Council.
 # Original work Copyright (c) 1999-2008 Pearu Peterson.
 
@@ -117,16 +117,16 @@ master_doc = 'index'
 
 # General information about the project.
 project = 'fparser'
-copyright = '2017-2019, Science and Technology Facilities Council'
+copyright = '2017-2020, Science and Technology Facilities Council'
 
 # The version info for the project you're documenting, acts as replacement for
 # |version| and |release|, also used in various other places throughout the
 # built documents.
 #
 # The short X.Y version.
-version = '0.0.10'
+version = '0.0.11'
 # The full version, including alpha/beta/rc tags.
-release = '0.0.10'
+release = '0.0.11'
 
 # The language for content autogenerated by Sphinx. Refer to documentation
 # for a list of supported languages.
@@ -200,7 +200,7 @@ html_theme = 'default'
 # Add any paths that contain custom static files (such as style sheets) here,
 # relative to this directory. They are copied after the builtin static files,
 # so a file named "default.css" will overwrite the builtin "default.css".
-html_static_path = ['_static']
+# html_static_path = ['_static']
 
 # Add any extra paths that contain custom files (such as robots.txt or
 # .htaccess) here, relative to this directory. These files are copied
@@ -270,7 +270,7 @@ latex_elements = {
 #  author, documentclass [howto, manual, or own class]).
 latex_documents = [
   ('index', 'fparser.tex', 'fparser Documentation',
-   'Andrew Porter, Rupert Ford and Pearu Peterson', 'manual'),
+   'Andrew Porter, Rupert Ford, Balthasar Reuter and Pearu Peterson', 'manual'),
 ]
 
 # The name of an image file (relative to this directory) to place at the top of
@@ -300,7 +300,7 @@ latex_documents = [
 # (source start file, name, description, authors, manual section).
 man_pages = [
     ('index', 'fparser', 'fparser Documentation',
-     ['Andrew Porter, Rupert Ford and Pearu Peterson'], 1)
+     ['Andrew Porter, Rupert Ford, Balthasar Reuter and Pearu Peterson'], 1)
 ]
 
 # If true, show URL addresses after external links.
@@ -314,7 +314,7 @@ man_pages = [
 #  dir menu entry, description, category)
 texinfo_documents = [
   ('index', 'fparser', 'fparser Documentation',
-   'Andrew Porter, Rupert Ford and Pearu Peterson', 'fparser', 'One line description of project.',
+   'Andrew Porter, Rupert Ford, Balthasar Reuter and Pearu Peterson', 'fparser', 'One line description of project.',
    'Miscellaneous'),
 ]
 
