@@ -1,6 +1,6 @@
 #!/usr/bin/env python
 # -*- coding: utf-8 -*-
-# Modified work Copyright (c) 2017-2019 Science and Technology
+# Modified work Copyright (c) 2017-2020 Science and Technology
 # Facilities Council
 # Original work Copyright (c) 1999-2008 Pearu Peterson
 
@@ -210,6 +210,50 @@ _HOLLERITH_START_SEARCH = re.compile(r'(?P<pre>\A|,\s*)'
 _IS_CALL_STMT = re.compile(r'call\b', re.I).match
 
 
+def extract_label(line):
+    '''Look for an integer label at the start of 'line' and if there is
+    one then remove it from 'line' and store it as an integer in
+    'label', returning both in a tuple.
+
+    :param str line: a string that potentially contains a label at the \
+        start.
+
+    :returns: a 2-tuple containing the label and updated line if a \
+        label is found or None and the unchanged line if a label is \
+        not found.
+    :rtype: (int or NoneType, str)
+
+    '''
+    label = None
+    match = _LABEL_RE.match(line)
+    if match:
+        label = int(match.group('label'))
+        line = line[match.end():].lstrip()
+    return label, line
+
+
+def extract_construct_name(line):
+    '''Look for a construct name at the start of 'line' and if there is
+    one then remove it from 'line' and return it as a string in
+    'name', returning both in a tuple.
+
+    :param str line: a string that potentially contains a construct \
+        name at the start.
+
+    :returns: a 2-tuple containing the construct name and updated line \
+        if a construct name is found or None and the unchanged line if \
+        a construct name is not found.
+    :rtype: (str or NoneType, str)
+
+    '''
+    construct_name = None
+    match = _CONSTRUCT_NAME_RE.match(line)
+    if match:
+        construct_name = match.group('name')
+        line = line[match.end():].lstrip()
+    return construct_name, line
+
+
 class FortranReaderError(Exception):
     '''
     Thrown when there is an error reading the Fortran source file.
@@ -471,8 +515,26 @@ class SyntaxErrorMultiLine(MultiLine, FortranReaderError):
         FortranReaderError.__init__(self, message)
 
 
+class CppDirective(Line):
+    '''Holds a preprocessor directive source line.
+
+    :param str line: string containing the text of a single or \
+                     multi-line preprocessor directive.
+    :param linenospan: a 2-tuple containing the start and end line \
+                       numbers of the directive from the input source.
+    :type linenospan: (int, int)
+    :param reader: The reader object being used to read the input \
+                   source.
+    :type reader: :py:class:`fparser.common.readfortran.FortranReaderBase`
+
+    '''
+    def __init__(self, line, linenospan, reader):
+        super(CppDirective, self).__init__(
+            line, linenospan, None, None, reader)
+
 ##############################################################################
 
+
 class FortranReaderBase(object):
     """
     Base class for reading Fortran sources.
@@ -838,10 +900,35 @@ class FortranReaderBase(object):
                    and ';' in item.get_line():
                 # ;-separator not recognized in pyf-mode
                 items = []
-                for line in item.get_line().split(';'):
+                # Deal with each Fortran statement separately.
+                split_line_iter = iter(item.get_line().split(';'))
+                first = next(split_line_iter)
+                # The full line has already been processed as a Line
+                # object in 'item' (and may therefore have label
+                # and/or construct name properties extracted from the
+                # start of the line). The simplest way to avoid losing
+                # any label or construct name properties for the first
+                # statement is to copy the 'item' object and update it
+                # so that it only includes text for the first
+                # statement (rather than the full line). Subsequent
+                # statements need to be processed into Line
+                # objects.
+                items.append(item.copy(first.strip(), apply_map=True))
+                for line in split_line_iter:
+                    # Any subsequent statements have not been processed
+                    # before, so new Line objects need to be created.
                     line = line.strip()
                     if line:
-                        items.append(item.copy(line, apply_map=True))
+                        # The statement might have a label and/or construct
+                        # name.
+                        label, line = extract_label(line)
+                        name, line = extract_construct_name(line)
+                        # Create a new Line object and append to items
+                        # using the existing span (line numbers) and
+                        # reader.
+                        new_line = Line(item.apply_map(line), item.span, label,
+                                        name, item.reader)
+                        items.append(new_line)
                 items.reverse()
                 for newitem in items:
                     self.fifo_item.insert(0, newitem)
@@ -886,6 +973,20 @@ class FortranReaderBase(object):
         """
         return Comment(comment, (startlineno, endlineno), self)
 
+    def cpp_directive_item(self, line, startlineno, endlineno):
+        '''
+        Construct :py:class:`fparser.common.readfortran.CppDirective` item.
+
+        :param str line: string containing the text of a single or \
+                         multi-line preprocessor directive.
+        :param int startlineno: start line number of the directive from \
+                                the input source.
+        :param int endlineno: end line number of the directive from \
+                              the input source.
+
+        '''
+        return CppDirective(line, (startlineno, endlineno), self)
+
     # For handling messages:
 
     def format_message(self, kind, message, startlineno, endlineno,
@@ -978,6 +1079,30 @@ class FortranReaderBase(object):
 
     # Auxiliary methods for processing raw source lines:
 
+    def handle_cpp_directive(self, line):
+        '''
+        Determine whether the current line is likely to hold
+        C preprocessor directive.
+
+        If the first non whitespace character of a line is the symbol ``#``
+        it is assumed this is a preprocessor directive.
+
+        Preprocessor directives can be used only in Fortran codes. They are
+        ignored when used inside PYF files.
+
+        The actual line content is not altered.
+
+        :param str line: the line to be tested for directives.
+
+        :return: a tuple containing the line and True/False depending on \
+                 whether it holds a C preprocessor directive.
+        :rtype: (str, bool)
+
+        '''
+        if not line or self._format.is_pyf:
+            return (line, False)
+        return (line, line.lstrip().startswith('#'))
+
     def handle_cf2py_start(self, line):
         """ Apply f2py directives to line.
 
@@ -1152,14 +1277,28 @@ class FortranReaderBase(object):
         A source item is ..
         - a fortran line
         - a list of continued fortran lines
-        - a multiline - lines inside triple-qoutes, only when in ispyf mode
+        - a multiline - lines inside triple-quotes, only when in ispyf mode
         - a comment line
+        - a preprocessor directive line
         """
         get_single_line = self.get_single_line
         line = get_single_line()
         if line is None:
             return
         startlineno = self.linecount
+        line, is_cpp_directive = self.handle_cpp_directive(line)
+        if is_cpp_directive:
+            # CPP directive line
+            lines = []
+            while line.rstrip().endswith('\\'):
+                # Line continuation
+                lines.append(line.rstrip()[:-1])
+                line = get_single_line()
+            lines.append(line)
+            endlineno = self.linecount
+            return self.cpp_directive_item(''.join(lines), startlineno,
+                                           endlineno)
+
         line = self.handle_cf2py_start(line)
         is_f2py_directive = startlineno in self.f2py_comment_lines
         isstrict = self._format.is_strict
@@ -1355,20 +1494,13 @@ class FortranReaderBase(object):
                         line = get_single_line()
                         continue
                 else:
-                    # first line, check for a label
-                    m = _LABEL_RE.match(line)
-                    if m:
-                        assert not label, repr(label)
-                        label = int(m.group('label'))
-                        line = line[m.end():]
-                    # check for a construct name
-                    m = _CONSTRUCT_NAME_RE.match(line)
-                    if m:
-                        name = m.group('name')
-                        line = line[m.end():].lstrip()
-                line, qc, had_comment = handle_inline_comment(line,
-                                                              self.linecount,
-                                                              qc)
+                    # Extract label and/or construct name from line if
+                    # there is one.
+                    label, line = extract_label(line)
+                    name, line = extract_construct_name(line)
+
+                line, qc, had_comment = \
+                    handle_inline_comment(line, self.linecount, qc)
                 have_comment |= had_comment
                 is_f2py_directive = self.linecount in self.f2py_comment_lines
 
