@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 ##############################################################################
-# Copyright (c) 2017-2019 Science and Technology Facilities Council
+# Copyright (c) 2017-2020 Science and Technology Facilities Council
 #
 # All rights reserved.
 #
@@ -45,19 +45,20 @@ from __future__ import print_function
 import io
 import os.path
 import tempfile
-import six
 import re
+import six
 import pytest
 
-from fparser.common.readfortran import FortranFileReader, FortranStringReader
-import fparser.common.sourceinfo
-import fparser.common.tests.logging_utils
+from fparser.common.readfortran import FortranFileReader, \
+    FortranStringReader, FortranReaderBase, Line, extract_label, \
+    extract_construct_name, CppDirective
+from fparser.common.sourceinfo import FortranFormat
 
 
 def test_empty_line_err():
     ''' Check that we raise the expected error if we try and create
     an empty Line '''
-    from fparser.common.readfortran import Line, FortranReaderError
+    from fparser.common.readfortran import FortranReaderError
     with pytest.raises(FortranReaderError) as err:
         _ = Line("   ", 1, "", "a_name", None)
     assert "Got empty line: \'   \'" in str(err.value)
@@ -90,10 +91,8 @@ def test_111fortranreaderbase(log, monkeypatch):
 
     monkeypatch.setattr('fparser.common.readfortran.FortranReaderBase.id',
                         lambda x: 'foo', raising=False)
-    mode = fparser.common.sourceinfo.FortranFormat(True, False)
-    unit_under_test = fparser.common.readfortran.FortranReaderBase(FailFile(),
-                                                                   mode,
-                                                                   True)
+    mode = FortranFormat(True, False)
+    unit_under_test = FortranReaderBase(FailFile(), mode, True)
     assert str(unit_under_test.next()) == "line #1'x=1'"
     with pytest.raises(StopIteration):
         unit_under_test.next()
@@ -114,7 +113,7 @@ def test_include_not_found():
 
     '''
     code = "include 'nonexistant.f90'"
-    unit_under_test = fparser.common.readfortran.FortranStringReader(code)
+    unit_under_test = FortranStringReader(code)
     line = unit_under_test.next()
     assert str(line.line) == code
 
@@ -126,9 +125,8 @@ def test_base_next_good_include(log):
     '''
     code = "include 'modfile.f95'\nx=2"
     include_directories = [os.path.dirname(__file__)]
-    unit_under_test = fparser.common.readfortran \
-        .FortranStringReader(code, include_dirs=include_directories,
-                             ignore_comments=False)
+    unit_under_test = FortranStringReader(
+        code, include_dirs=include_directories, ignore_comments=False)
     line = unit_under_test.next()
     assert str(line)[:19] == "Comment('! Modified"  # First line of inclusion
     assert log.messages['debug'] == []
@@ -146,7 +144,7 @@ def test_fortranreaderbase_info(log):
     '''
     Tests that FortranReaderBase.info() causes a message to be logged.
     '''
-    unit_under_test = fparser.common.readfortran.FortranStringReader('x=3')
+    unit_under_test = FortranStringReader('x=3')
     thing = unit_under_test.get_source_item()
     unit_under_test.info('Mighty Whirlitzer', thing)
     assert log.messages['debug'] == []
@@ -162,7 +160,7 @@ def test_fortranreaderbase_error(log):
     '''
     Tests that FortranReaderBase.error() causes a message to be logged.
     '''
-    unit_under_test = fparser.common.readfortran.FortranStringReader('x=2')
+    unit_under_test = FortranStringReader('x=2')
     thing = unit_under_test.get_source_item()
     with pytest.raises(SystemExit):
         unit_under_test.error('Thundering Chalmer', thing)
@@ -179,7 +177,7 @@ def test_fortranreaderbase_warning(log):
     '''
     Tests that FortranReaderBase.warning() causes a message to be logged.
     '''
-    unit_under_test = fparser.common.readfortran.FortranStringReader('x=1')
+    unit_under_test = FortranStringReader('x=1')
     thing = unit_under_test.get_source_item()
     unit_under_test.warning('Flatulent Hermit', thing)
     assert log.messages['debug'] == []
@@ -198,8 +196,8 @@ def test_base_handle_multilines(log):
     '''
     code = 'character(8) :: test = \'foo"""bar'
     log.reset()
-    unit_under_test = fparser.common.readfortran.FortranStringReader(code)
-    mode = fparser.common.sourceinfo.FortranFormat(True, True)
+    unit_under_test = FortranStringReader(code)
+    mode = FortranFormat(True, True)
     unit_under_test.set_format(mode)  # Force strict free format
     unit_under_test.get_source_item()
     assert log.messages['debug'] == []
@@ -212,8 +210,8 @@ def test_base_handle_multilines(log):
 
     code = 'goo """boo\n doo""" soo \'foo'
     log.reset()
-    unit_under_test = fparser.common.readfortran.FortranStringReader(code)
-    mode = fparser.common.sourceinfo.FortranFormat(True, True)
+    unit_under_test = FortranStringReader(code)
+    mode = FortranFormat(True, True)
     unit_under_test.set_format(mode)  # Force strict free format
     unit_under_test.get_source_item()
     assert log.messages['debug'] == []
@@ -234,8 +232,8 @@ def test_base_fixed_nonlabel(log):
     # logged.
     code = 'w    integer :: i'
     log.reset()
-    unit_under_test = fparser.common.readfortran.FortranStringReader(code)
-    mode = fparser.common.sourceinfo.FortranFormat(False, True)
+    unit_under_test = FortranStringReader(code)
+    mode = FortranFormat(False, True)
     unit_under_test.set_format(mode)  # Force fixed format
     unit_under_test.get_source_item()
     assert log.messages['debug'] == []
@@ -251,8 +249,8 @@ def test_base_fixed_nonlabel(log):
     for i in range(1, 5):
         code = ' '*i + 'w' + ' '*(5-i) + 'integer :: i'
         log.reset()
-        unit_under_test = fparser.common.readfortran.FortranStringReader(code)
-        mode = fparser.common.sourceinfo.FortranFormat(False, True)
+        unit_under_test = FortranStringReader(code)
+        mode = FortranFormat(False, True)
         unit_under_test.set_format(mode)  # Force strict fixed format
         unit_under_test.get_source_item()
         assert log.messages['debug'] == []
@@ -268,8 +266,8 @@ def test_base_fixed_nonlabel(log):
     # engaged.
     code = ' w   integer :: i'
     log.reset()
-    unit_under_test = fparser.common.readfortran.FortranStringReader(code)
-    mode = fparser.common.sourceinfo.FortranFormat(False, False)
+    unit_under_test = FortranStringReader(code)
+    mode = FortranFormat(False, False)
     unit_under_test.set_format(mode)  # Force sloppy fixed format
     unit_under_test.get_source_item()
     assert log.messages['debug'] == []
@@ -289,8 +287,8 @@ def test_base_fixed_continuation(log):
     '''
     code = '     character(4) :: cheese = "a & !\n     & b'
     log.reset()
-    unit_under_test = fparser.common.readfortran.FortranStringReader(code)
-    mode = fparser.common.sourceinfo.FortranFormat(False, False)
+    unit_under_test = FortranStringReader(code)
+    mode = FortranFormat(False, False)
     unit_under_test.set_format(mode)  # Force sloppy fixed format
     unit_under_test.get_source_item()
     assert log.messages['debug'] == []
@@ -303,8 +301,8 @@ def test_base_fixed_continuation(log):
 
     code = '     x=1 &\n     +1 &\n     -2'
     log.reset()
-    unit_under_test = fparser.common.readfortran.FortranStringReader(code)
-    mode = fparser.common.sourceinfo.FortranFormat(False, False)
+    unit_under_test = FortranStringReader(code)
+    mode = FortranFormat(False, False)
     unit_under_test.set_format(mode)  # Force sloppy fixed format
     unit_under_test.get_source_item()
     assert log.messages['debug'] == []
@@ -324,8 +322,8 @@ def test_base_free_continuation(log):
     '''
     code = 'character(4) :: "boo & que'
     log.reset()
-    unit_under_test = fparser.common.readfortran.FortranStringReader(code)
-    mode = fparser.common.sourceinfo.FortranFormat(True, False)
+    unit_under_test = FortranStringReader(code)
+    mode = FortranFormat(True, False)
     unit_under_test.set_format(mode)  # Force sloppy free format
     unit_under_test.get_source_item()
     assert log.messages['debug'] == []
@@ -509,6 +507,86 @@ def test_include6(tmpdir, ignore_comments):
                         expected, tmpdir, ignore_comments=ignore_comments)
 
 
+def test_multi_stmt_line1():
+    '''Check that simple statements separated by ; on a single line are
+    correctly split into multiple lines by FortranReaderBase
+
+    '''
+    code = "do i=1,10;b=20 ; c=30"
+    reader = FortranStringReader(code)
+    mode = FortranFormat(True, False)
+    reader.set_format(mode)
+    line1 = reader.next()
+    assert isinstance(line1, Line)
+    assert line1.line == "do i=1,10"
+    assert line1.span == (1, 1)
+    assert line1.label is None
+    assert line1.name is None
+    assert line1.reader is reader
+    line2 = reader.next()
+    assert isinstance(line2, Line)
+    assert line2.line == "b=20"
+    assert line2.span is line1.span
+    assert line2.label is None
+    assert line2.name is None
+    assert line2.reader is reader
+    line3 = reader.next()
+    assert isinstance(line3, Line)
+    assert line3.line == "c=30"
+    assert line3.span is line1.span
+    assert line3.label is None
+    assert line3.name is None
+    assert line3.reader is reader
+
+
+def test_multi_stmt_line2():
+    '''Check that format statements separated by ; on a single line are
+    correctly split into multiple lines by FortranReaderBase
+
+    '''
+    code = "10 format(a); 20 format(b)"
+    reader = FortranStringReader(code)
+    mode = FortranFormat(True, False)
+    reader.set_format(mode)
+    line1 = reader.next()
+    assert isinstance(line1, Line)
+    assert line1.line == "format(a)"
+    assert line1.span == (1, 1)
+    assert line1.label == 10
+    assert line1.name is None
+    assert line1.reader is reader
+    line2 = reader.next()
+    assert line2.line == "format(b)"
+    assert line2.span is line1.span
+    assert line2.label == 20
+    assert line2.name is None
+    assert line2.reader is reader
+
+
+def test_multi_stmt_line3():
+    '''Check that named do loops separated by ; on a single line are correctly
+    split into multiple lines by FortranReaderBase
+
+    '''
+    code = "name:do i=1,10;name2 : do j=1,10"
+    reader = FortranStringReader(code)
+    mode = FortranFormat(True, False)
+    reader.set_format(mode)
+    line1 = reader.next()
+    assert isinstance(line1, Line)
+    assert line1.line == "do i=1,10"
+    assert line1.span == (1, 1)
+    assert line1.label is None
+    assert line1.name == "name"
+    assert line1.reader is reader
+    line2 = reader.next()
+    assert line2.line == "do j=1,10"
+    assert line2.span is line1.span
+    assert line2.label is None
+    assert line2.name == "name2"
+    assert line2.reader is reader
+
+
 def test_get_item(ignore_comments):
     '''Check the get_item() function works as expected. Test with and
     without comments being ignored.
@@ -632,7 +710,7 @@ def test_filename_reader():
             source_file.write(FULL_FREE_SOURCE)
 
         unit_under_test = FortranFileReader(filename)
-        expected = fparser.common.sourceinfo.FortranFormat(True, False)
+        expected = FortranFormat(True, False)
         assert unit_under_test.format == expected
         for expected in FULL_FREE_EXPECTED:
             found = unit_under_test.get_single_line(ignore_empty=True)
@@ -657,7 +735,7 @@ def test_file_reader():
         with io.open(filename, mode='r', encoding='UTF-8') as source_file:
             unit_under_test = FortranFileReader(source_file)
 
-            expected = fparser.common.sourceinfo.FortranFormat(True, False)
+            expected = FortranFormat(True, False)
             assert unit_under_test.format == expected
             for expected in FULL_FREE_EXPECTED:
                 assert unit_under_test.get_single_line(ignore_empty=True) \
@@ -675,7 +753,7 @@ def test_bad_file_reader():
     which is neither file nor filename.
     '''
     with pytest.raises(ValueError) as ex:
-        unit_under_test = FortranFileReader(42)
+        _ = FortranFileReader(42)
     expected = 'FortranFileReader is used with a filename or file-like object.'
     assert expected in str(ex.value)
 
@@ -687,7 +765,7 @@ def test_string_reader():
     Tests that Fortran source can be read from a string.
     '''
     unit_under_test = FortranStringReader(FULL_FREE_SOURCE)
-    expected = fparser.common.sourceinfo.FortranFormat(True, False)
+    expected = FortranFormat(True, False)
     assert unit_under_test.format == expected
     for expected in FULL_FREE_EXPECTED:
         assert unit_under_test.get_single_line(ignore_empty=True) == expected
@@ -720,7 +798,7 @@ cf2py call me ! hey
                 "line #10'end'"]
 
     # Reading from buffer
-    reader = fparser.common.readfortran.FortranStringReader(
+    reader = FortranStringReader(
         string_f77, ignore_comments=False)
     assert reader.format.mode == 'f77', repr(reader.format.mode)
     stack = expected[:]
@@ -733,7 +811,7 @@ cf2py call me ! hey
     with open(filename, 'w') as fortran_file:
         print(string_f77, file=fortran_file)
 
-    reader = fparser.common.readfortran.FortranFileReader(
+    reader = FortranFileReader(
         filename, ignore_comments=False)
     stack = expected[:]
     for item in reader:
@@ -804,7 +882,7 @@ end python module foo
                 "line #25'end python module foo'",
                 "Comment('! end of file',(26, 26))"]
 
-    reader = fparser.common.readfortran.FortranStringReader(
+    reader = FortranStringReader(
         string_pyf, ignore_comments=False)
     assert reader.format.mode == 'pyf', repr(reader.format.mode)
     for item in reader:
@@ -854,7 +932,7 @@ cComment
                 "line #15'subroutine foo'",
                 "Comment('',(16, 16))",
                 "line #18'end'"]
-    reader = fparser.common.readfortran.FortranStringReader(
+    reader = FortranStringReader(
         string_fix90, ignore_comments=False)
     assert reader.format.mode == 'fix', repr(reader.format.mode)
     for item in reader:
@@ -874,3 +952,92 @@ def test_utf_char_in_code(log):
     while out_line:
         out_line = reader.get_item()
     assert log.messages['critical'] == []
+
+
+def test_extract_label():
+    ''' Test the extract label function in readfortran.py.'''
+    text_input = "no label"
+    label, text_result = extract_label(text_input)
+    assert label is None
+    assert text_result is text_input
+
+    text_input = " 80stuff"
+    label, text_result = extract_label(text_input)
+    assert label is None
+    assert text_result is text_input
+
+    text_input = " 80 stuff"
+    label, text_result = extract_label(text_input)
+    assert label == 80
+    assert text_result == "stuff"
+
+
+def test_extract_construct_name():
+    '''Test the extract construct name function in readfortran.py.'''
+    text_input = "no construct name"
+    name, text_result = extract_construct_name(text_input)
+    assert name is None
+    assert text_result is text_input
+
+    text_input = "name:stuff"
+    name, text_result = extract_construct_name(text_input)
+    assert name == "name"
+    assert text_result == "stuff"
+
+    text_input = " name : stuff"
+    name, text_result = extract_construct_name(text_input)
+    assert name == "name"
+    assert text_result == "stuff"
+
+
+@pytest.mark.parametrize('input_text, ref', [
+    ('#include "abc"', True),
+    (' #include "abc"', True),
+    (' #  include "abc"', True),
+    ('#define ABC 1', True),
+    ('#ifdef ABC', True),
+    ('#if !defined(ABC)', True),
+    ('abc #define', False),
+    ('"#"', False),
+    ('! #', False)])
+def test_handle_cpp_directive(input_text, ref):
+    '''Test the function that detects cpp directives in readfortran.py.'''
+    reader = FortranStringReader(input_text)
+    output_text, result = reader.handle_cpp_directive(input_text)
+    assert result == ref
+    assert output_text is input_text
+
+
+def test_reader_cpp_directives():
+    '''Test that CPP directives are read correctly in readfortran.py.'''
+    input_text = [
+        'program test', '#define ABC 123', "character :: c = '#'",
+        '#ifdef ABC', '! Some comment that should be ignored', 'integer :: a',
+        '#endif', '#if !defined(ABC)', 'integer :: b', '#endif',
+        'end program test']
+    ref_text = '\n'.join(input_text[:4] + input_text[5:]).strip()
+    reader = FortranStringReader('\n'.join(input_text))
+    lines = [item for item in reader]
+
+    pp_lines = [1, 3, 5, 6, 8]
+    for i, line in enumerate(lines):
+        if i in pp_lines:
+            assert isinstance(line, CppDirective)
+        else:
+            assert isinstance(line, Line)
+
+    assert '\n'.join(item.line for item in lines) == ref_text
+
+
+def test_multiline_cpp_directives():
+    '''Test that multiline CPP directives are read correctly.'''
+    input_text = [
+        'program test', '#define ABC \\ ', '  123 ',
+        'integer a', 'end program test']
+    ref_text = 'program test\n#define ABC   123\ninteger a\nend program test'
+    reader = FortranStringReader('\n'.join(input_text))
+    lines = [item for item in reader]
+    assert len(lines) == 4
+    assert isinstance(lines[1], CppDirective)
+    assert lines[1].span == (2, 3)
+    assert '\n'.join(item.line for item in lines) == ref_text
