@@ -810,7 +810,8 @@ class EndSubroutine(EndStatement):
     """
     END [SUBROUTINE [name]]
     """
-    match = re.compile(r'end(\s*subroutine\s*\w*|)\Z', re.I).match
+    match = re.compile(r'end\s*(?:subroutine\s*(?:(?P<name>\w+)\s*)?)?$',
+                       re.IGNORECASE).match
 
 
 class Subroutine(SubProgramStatement):
@@ -1145,9 +1146,60 @@ class EndDo(EndStatement):
     """
     END DO [<do-construct-name>]
     """
-    match = re.compile(r'end\s*do\s*\w*\Z', re.I).match
+    match = re.compile(r'end\s*do\s*(?:(?P<name>\w+)\s*)?\Z',
+                       re.IGNORECASE).match
     blocktype = 'do'
 
+    def process_item(self):
+        '''
+        Parses the next line assuming it is an "End do" statement.
+
+        Overrides method in `EndStatement`.
+        '''
+        item = self.item
+        line = item.get_line()
+        matched = self.match(line)
+        # Check for matching labels
+        found_label = getattr(self.item, 'label', None)
+        expected_label = getattr(self.parent, 'endlabel', None)
+        if expected_label:
+            if found_label:
+                if found_label != expected_label:
+                    message = 'When entering the "do" block {start} was' \
+                              + ' given as the end label but {end} was found.'
+                    self.warning(message.format(start=expected_label,
+                                                end=found_label))
+                    self.isvalid = False
+            else:
+                message = 'A label was specified when entering the "do" ' \
+                          + ' block ({label}) but none was found at the end.'
+                self.warning(message.format(label=expected_label))
+                self.isvalid = False
+        # Check for matching names
+        found_name = matched.group('name') or None
+        expected_name = self.parent.construct_name
+        if expected_name:
+            if found_name:
+                if found_name != expected_name:
+                    message = 'The "do" block was specified with the name' \
+                              + ' "{open}" but was closed with the name' \
+                              + ' "{close}".'
+                    self.warning(message.format(open=expected_name,
+                                                close=found_name))
+                    self.isvalid = False
+            else:
+                message = 'A name ("{name}") was specified for the "do" ' \
+                          + 'block but was not given when closing the block.'
+                self.warning(message.format(name=expected_name))
+                self.isvalid = False
+        else:
+            if found_name:
+                message = 'The name "{name}" was used when closing a "do"' \
+                          + 'block but none was specified when opening it.'
+                self.warning(message.format(name=found_name))
+                self.isvalid = False
+        return EndStatement.process_item(self)
+
 
 class Do(BeginStatement):
     """
@@ -1157,8 +1209,8 @@ class Do(BeginStatement):
     """
 
     match = re.compile(r'do\b\s*\d*', re.I).match
-    pattern = r'do\b\s*(?P<label>\d*)\s*,?\s*(?P<loopcontrol>.*)\Z'
-    item_re = re.compile(pattern, re.I).match
+    pattern = r'do\b\s*(?:(?P<label>\d+)\s*)?(?:,\s*)?(?P<loopcontrol>.+)?\Z'
+    item_re = re.compile(pattern, re.IGNORECASE).match
     end_stmt_cls = EndDo
     name = ''
 
@@ -1170,28 +1222,37 @@ class Do(BeginStatement):
         return ' '.join(lst)
 
     def process_item(self):
+        '''
+        Parses the next line assuming it is a "Do" statement.
+
+        Overrides method in `BeginStatement`.
+        '''
         item = self.item
         line = item.get_line()
-        m = self.item_re(line)
-        label = m.group('label').strip() or None
-        if label:
-            label = int(label)
-        self.endlabel = label
+        matched = self.item_re(line)
+        if matched.group('label'):
+            self.endlabel = int(matched.group('label').strip())
+        else:
+            self.endlabel = None
         self.construct_name = item.name
-        self.loopcontrol = m.group('loopcontrol').strip()
+        if matched.group('loopcontrol'):
+            self.loopcontrol \
+                = item.apply_map(matched.group('loopcontrol').strip())
+        else:
+            self.loopcontrol = None
         return BeginStatement.process_item(self)
 
     def process_subitem(self, item):
-        r = False
+        result = False
         if self.endlabel:
             label = item.label
             if label == self.endlabel:
-                r = True
+                result = True
                 if isinstance(self.parent,
                               Do) and label == self.parent.endlabel:
                     # the same item label may be used for different block ends
                     self.put_item(item)
-        return BeginStatement.process_subitem(self, item) or r
+        return BeginStatement.process_subitem(self, item) or result
 
     def get_classes(self):
         return execution_part_construct
