@@ -687,9 +687,10 @@ class Allocate(Statement):
         :return: Fortran code
         :rtype: str
         '''
+        import six
         type_spec = ''
         if self.spec:
-            if isinstance(self.spec, str):
+            if isinstance(self.spec, six.text_type):
                 type_spec = self.spec
             else:
                 type_spec = self.spec.tostr()
